<?php

namespace Drupal\commerce_product_quantity\EventSubscriber;

use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_cart\Event\CartEntityAddEvent;
use Drupal\commerce_cart\Event\CartEvents;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Cart Event Subscriber Manages product limit within the cart.
 */
class CartEventSubscriber implements EventSubscriberInterface {

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The cart manager.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;

  public function __construct(MessengerInterface $messenger, CartManagerInterface $cart_manager) {
    $this->messenger = $messenger;
    $this->cartManager = $cart_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getEvents() {
    return [
      CartEvents::CART_ENTITY_ADD => [['displayAddToCartMessage', -100]],
    ];
  }

  /**
   * Displays an add to cart message The add to cart event.
   *
   * @param \Drupal\commerce_cart\Event\CartEntityAddEvent $event
   *   The event that triggered the add to cart message.
   */
  public function displayAddToCartMessage(CartEntityAddEvent $event) {
    $this->messenger->addMessage($this->t('@entity added to <a href=":url">your cart</a>.', [
      '@entity' => $event->getEntity()->label(),
      ':url' => Url::fromRoute('commerce_cart.page')->toString(),
    ]));
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      CartEvents::CART_ENTITY_ADD => [['addToCart', -100]],
    ];
  }

  /**
   * Add a related product automatically.
   *
   * @param \Drupal\commerce_cart\Event\CartEntityAddEvent $event
   *   The cart add event.
   *
   * @throws \Drupal\Core\TypedData\Exception\ReadOnlyException
   */
  public function addToCart(CartEntityAddEvent $event) {
    /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $product_variation */

    /*
     * Get stored quantity and product ID from config
     */
    $config = \Drupal::config('commerce_product_quantity.settings');

    /*
     * $product_count will get the count of listed.
     *
     * Products contain the quantity.
     */
    $product_count = $config->get('product_quantity_count');

    /*
     *  If isset $product_count then this condition will call.
     */
    if ($product_count) {
      /*
       * $product_list will store product and it's quantity
       *
       * Saved in the 'admin/config/system/commerce_product_quantity'
       * products configuration.
       */
      $product_list = [];
      for ($i = 0; $i < $product_count; $i++) {
        $product_list[$i]['product'] = $config->get('product_' . $i);
        $product_list[$i]['quantity'] = $config->get('quantity_' . $i);
      }

      $cart = $event->getCart();
      $cart_items = $cart->getItems();

      foreach ($cart_items as $cart_item) {
        $quantity_flag = $number_item = $default_quantity = 0;
        $quantity = $cart_item->getQuantity();
        $temp_product_variation = $cart_item->getPurchasedEntity();
        $product_id = $temp_product_variation->product_id->getString();
        $product_key = array_search($product_id, array_column($product_list, 'product'));
        if (isset($product_key) && !empty($product_key)) {
          $quantity_flag = $quantity_flag + $quantity;
          $number_item = $number_item + 1;
          $default_quantity = $product_list[$product_key]['quantity'];
        }
        if ($quantity_flag > $default_quantity) {
          if ($number_item == 1) {
            $cart_item->setQuantity($default_quantity);
            $this->messenger->deleteAll();
            $this->messenger->addMessage(t('You have reached max quantity for this product'), 'error');
          }
          else {
            $cart->removeItem($cart_item);
            $cart_item->delete();
            $this->messenger->deleteAll();
            $this->messenger->addMessage(t('You have reached max quantity for this product'), 'error');
          }
        }
      }
      $cart->save();
    }
    /*
     * If configuration is empty on.
     *
     * 'admin/config/system/commerce_product_quantity' Path then
     * below line will get called.
     */
    self::getEvents();
  }

}
