<?php

namespace Drupal\commerce_product_quantity\EventSubscriber;

use Drupal\commerce_cart\CartManagerInterface;
use Drupal\commerce_cart\Event\CartEvents;
use Drupal\commerce_cart\Event\CartOrderItemUpdateEvent;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Cart Event Subscriber Manages product limit within the cart.
 */
class CartUpdateEventSubscriber implements EventSubscriberInterface {

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The cart manager.
   *
   * @var \Drupal\commerce_cart\CartManagerInterface
   */
  protected $cartManager;

  public function __construct(MessengerInterface $messenger, CartManagerInterface $cart_manager) {
    $this->messenger = $messenger;
    $this->cartManager = $cart_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      CartEvents::CART_ORDER_ITEM_UPDATE => [
        ['updateInCart', -100],
      ],
    ];
  }

  /**
   * Add a related product automatically.
   *
   * @param \Drupal\commerce_cart\Event\CartOrderItemUpdateEvent $event
   *   The cart add event.
   *
   * @throws \Drupal\Core\TypedData\Exception\ReadOnlyException
   */
  public function updateInCart(CartOrderItemUpdateEvent $event) {
    /*
     * Get list of all selected products from configuration.
     *
     * And update according to in cart.
     */
    $config = \Drupal::config('commerce_product_quantity.settings');
    $product_count = $config->get('product_quantity_count');

    $product_list = [];
    for ($i = 0; $i < $product_count; $i++) {
      $product_list[$i]['product'] = $config->get('product_' . $i);
      $product_list[$i]['quantity'] = $config->get('quantity_' . $i);
    }

    /*
     * Get quantity for particular product_type.
     */
    $product_type_config = \Drupal::config('commerce_product_type_quantity.settings');
    $product_type_count = $product_type_config->get('product_quantity_count');

    $product_type_list = [];
    for ($i = 0; $i < $product_type_count; $i++) {
      $product_type_list[$i]['product_type'] = $product_type_config->get('product_' . $i);
      $product_type_list[$i]['quantity'] = $product_type_config->get('quantity_' . $i);
    }

    $cart = $event->getCart();
    $cart_items = $cart->getItems();
    $titles = [];
    foreach ($cart_items as $cart_item) {
      $quantity_flag = $number_item = $default_quantity = 0;
      $quantity = $cart_item->getQuantity();
      $temp_product_variation = $cart_item->getPurchasedEntity();
      $product_id = $temp_product_variation->product_id->getString();
      $p = explode(' ', $product_id);
      $product = \Drupal::entityTypeManager()->getStorage('commerce_product')->loadMultiple($p);
      foreach ($product as $products) {
        $titles = $products->bundle();
      }

      $product_key = array_search($product_id, array_column($product_list, 'product'));
      $product_type_key = array_search($titles, array_column($product_type_list, 'product_type'));
      $product_type = $product_type_list[$product_type_key]['quantity'];

      if (is_int($product_key)) {
        $quantity_flag = $quantity_flag + $quantity;
        $number_item = $number_item + 1;
        $default_quantity = intval($product_list[$product_key]['quantity']);
      }
      elseif (is_int($product_type_key)) {
        $quantity_flag = $quantity_flag + $quantity;
        $number_item = $number_item + 1;
        $default_quantity = intval($product_type_list[$product_type_key]['quantity']);
      }
      else {
        /*
         * Handle the scenario where the product is not found in the product.
         *
         * List Check if a default quantity is set, otherwise
         * remove the item from the cart.
         */
        if ($default_quantity !== NULL) {
          if ($default_quantity > 0) {
            $this->messenger->deleteAll();
            $this->messenger->addMessage(t('You have reached the maximum quantity for this product 000'), 'error');
          }
        }
        else {
          $cart->removeItem($cart_item);
          $cart_item->delete();
        }
        continue;
      }

      if ($default_quantity > 0 && $quantity_flag > $default_quantity) {
        /*
         * Update the cart item quantity to the default quantity.
         */
        $cart_item->setQuantity($default_quantity);
        $this->messenger->deleteAll();
        if (is_int($product_key)) {
          $this->messenger->addMessage(t('You have reached the maximum quantity for this product'), 'error');
        }
        else {
          $this->messenger->addMessage(t('You have reached the maximum quantity for this product type'), 'error');
        }
      }
    }
    $cart->save();
  }

}
